#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include "config.h"


//Definicje portow i pinu dla tyrystora
#define TYPORT PORTD
#define TYDIR DDRD
#define TYPIN _BV(PD3)

#define TY1() TYPORT|=TYPIN
#define TY0() TYPORT&=~TYPIN

//Definicje dla klawiatury
#define KEYPIN PINC
#define KEYDIR DDRC
#define KEYPORT PORTC
#define K1PIN _BV(PC1)
#define K2PIN _BV(PC2)
#define K3PIN _BV(PC3)

//Zmienna globalna timera
volatile u08 Tim;
//Zmienna globalna timera wyswietlacza
volatile u08 DispTim;
//Timer wydluzony do 32 bitowy
volatile u32 TimPid;
//Timer wyjsciowy do glownego menu
volatile u16 ExitTim;
//Timer zezwolen na klawiature
volatile u08 KeyZTim;
//Zmienna globalna wspolczynnika wypelnienia
volatile u08 pwm_p;
//Kod wcisnietego klawisza
volatile u08 key_p;
//Czy klawisz powtorzony
volatile u08 key_r;

//Zmienne dla przetwornika ac
static volatile u08 AdcNew;
static volatile u16 AdcVal;
//Inicjalizacja wszystkich ukladow peryferyjnych
void InitHw(void)
{
	//Timer 1 100 razy na sekunde (CTC)
	TCCR1A = 0;
	TCNT1 = 0;
	OCR1A = 1249;
	TCCR1B = _BV(WGM12) | _BV(CS11) | _BV(CS10);
	TIMSK |= _BV(OCIE1A);
	//Inicjalizacja przerwan zewn i tyrystora
	TYDIR |= TYPIN;
	MCUCR |= _BV(ISC00);		//Kazde zbocze gen IRQ
	MCUCR &= ~_BV(ISC01);
	GICR |= _BV(INT0);			//Wlacz Przerw zewn
	//Inicjalizacja klawiatury
	KEYDIR &= ~ (K1PIN | K2PIN | K3PIN);	//Inp
	KEYPORT |= K1PIN | K2PIN | K3PIN;		//pullup R
	//Inicjalizacja przetwornika A/C
	ADMUX = _BV(REFS1) | _BV(REFS0);
	ADCSRA = _BV(ADPS2) | _BV(ADPS1) |_BV(ADPS0) | _BV(ADEN) | _BV(ADIE);
	//Wlacz globalnie przerwania
	sei();
}


//Przerwanie wywolywane 100 razy na sekunde
SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	//Obsluga Timera
	if(Tim) --Tim;
	//Obsluga Timera Wyswietlania
	if(DispTim) --DispTim;
	//Timer dlugi
	if(TimPid) --TimPid;
	//Timer wyjscia z menu
	if(ExitTim) --ExitTim;
	//Timer zezwolen na klawiature
	if(KeyZTim) --KeyZTim;
	//Obsluga klawiatury
	static u08 LastKey=0;
	static u08 RepeatK=0;
	static u08 DelayK=0;
	u08 PortKey;
	
	if(++RepeatK == 10)
	{
		RepeatK = 0;
		PortKey = (~KEYPIN >> 1)&0x07;
		if(PortKey)
		{
			if(PortKey == LastKey)
			{
				if(!DelayK)
				{	
					key_p = PortKey;
					key_r = 1;
				}
				else
				{
					--DelayK;
				}
			}
			else
			{
				key_p = LastKey = PortKey;
				key_r = 0;
				DelayK = 15;
			}
		}
		else
		{
			LastKey =0;
		}
	}
	//Start przetwarzania AC
	static u08 adcdiv = 0;
	if(++adcdiv>=5)
	{
		ADCSRA |= _BV(ADSC);
		adcdiv = 0;
	}
}

//Przerwanie zewnetrzne obsluga grupowej reg mocy
SIGNAL(SIG_INTERRUPT0)
{
	static u08 st_pos;
	static u08 pwm=0;
	if(pwm_p==0) {TY0(); return;}
	else if(pwm_p==0xff) {TY1(); return;}
	if(pwm)
	{
		if(!st_pos--)
		{
			pwm = !pwm;
			TY0();
			st_pos = 255u - pwm_p;
		}	
	}
	else
	{
		if(!st_pos--)
		{
			pwm = !pwm;
			TY1();
			st_pos = pwm_p;
		}
	}
}


/* Przerwanie od przetwornika A/C */
SIGNAL(SIG_ADC)
{
	static u08 cnt=0;
	static u16 val = 0;
	val += ADC;
	if(++cnt>=4)
	{
		AdcNew = 1;
		AdcVal = val>>2;
		cnt = 0;
		val = 0;
	}
}

/* Zwraca wartosc z przetwornika A/C
 * gdy brak wartosci zwraca -1
 */
int AdcGetValue(void)
{
	u16 val;
	if(!AdcNew) return -1;
	cli();
	val = AdcVal;
	AdcNew = 0;
	sei();
	return val;
}
