#define F_CPU 1000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "lcd.h"

#define INSTR 0x0
#define DANE 0x1


//Definicje pinow dla danych starszy polbajt
#define LCDDIRD DDRD
#define LCDOUTD PORTD
#define LCDIND PIND
//Definicje pinow mlodsza czesc
#define LCDDIRC DDRB
#define LCDOUTC PORTB


#define LCD_E  _BV(PB2)
#define LCD_RW _BV(PB1)
#define LCD_RS _BV(PB0)

#define E1() LCDOUTC|=LCD_E
#define E0() LCDOUTC&=~LCD_E
#define RW1() LCDOUTC|=LCD_RW
#define RW0() LCDOUTC&=~LCD_RW
#define RS1() LCDOUTC|=LCD_RS
#define RS0() LCDOUTC&=~LCD_RS


//Funkcja no operation
#define nop() asm volatile("nop"::)

//Funkcja zapisujaca dana 4 bitowa
static void wr4d(unsigned char n) 
{
	LCDOUTD &= 0x0F; 
	LCDOUTD |= n & 0xF0;
}



//Wysyla dana lub instrukcje w zaleznosci od id
void WriteLcd(unsigned char dana,unsigned char id)
{	
	unsigned char result;
	unsigned int tout = 16000;
	//Sprawdzenie czy jest wolny wyswietlacz
	do
	{
		E0();
		RS0();
		RW1();	
		LCDDIRD &= 0x0f;
		LCDOUTD |= 0xf0;
		nop();
		E1();
		nop();
		result = LCDIND;
		E0();
		nop();
		nop();
		E1();						//Opusc mlodszy polbajt
		nop();
		nop();
		E0();
		if(--tout == 0) return;
	}
	while(result & 0x80);
	LCDDIRD |= 0xF0;
	//Wlasciwy zapis danej do portu (Interfejs 4 bitowy)		
	E0();
	RW0();		
	if(id) RS1();
	else RS0();
	wr4d(dana);			//Starszy polbajt
	E1();
	nop();
	E0();
	wr4d(dana<<4);
	E1();
	nop();
	E0();
}
 

//Inicjalizacja modulu LCD
void InitLcd(void)
{
	LCDDIRD |= 0xF0;
	LCDDIRC |= LCD_E | LCD_RW | LCD_RS;
	LCDOUTC &= ~(LCD_E | LCD_RW | LCD_E);
	wr4d(LCD8BITS);
	E1();
	nop();
	E0();
	_delay_ms(20);
	E1();
	nop();
	E0();
	_delay_ms(10);
	LCDOUTC &= ~(LCD_E | LCD_RW | LCD_E);
	wr4d(LCD4BITS);
	E1();
	nop();
	E0();
	_delay_ms(20);
	WriteLcd(0x28,INSTR);		//Interfejs 4 bitowy 2 linie
	WriteLcd(0x6,INSTR);		//Tryb pracy
	WriteLcd(0x0c,INSTR);		//Wlacz wyswietlacz
	WriteLcd(LCDCLEAR,INSTR);	//Wyczysc wyswietlacz..
    _delay_ms(20);
}


//Wyswietla string na wyswietlaczu LCD (RAM)
void PutsLcd(char *nap)
{
	while(*nap)
	{   
		WriteLcd(*nap,DANE);
	    nap++;
	}	
}

//Wyswietla string na wyswietlaczu LCD (ROM)
void PutsLcd_P(PGM_P nap)
{
	while(pgm_read_byte(nap))
	{   
		WriteLcd(pgm_read_byte(nap),DANE);
	    nap++;
	}
}
