/**
LCD functions, header file.

*/
#ifndef LCD_H
#define LCD_H


#define LCDON         0x01 //0x00000001 //Switch on  display
#define LCDOFF        0x08 //0x00001000 //Switch off display




#define LCDCLEAR      0x01 //0x00000001

/** Place cursor on a specific display line. The horizontal offset
can be added: LCDLINEx + xoffset. As the line length is limited,
the horizontal offset must not be greater 64
*/
#define LCDLINE1      0x80 //0x10000000
#define LCDLINE2      0xc0 //0x11000000

/** Cursor modes.
*/
#define LCDCURSORON   0x0f //0x00001111 //turn on cursor blinking
#define LCDCURSOROFF  0x0c //0x00001100 //Disable cursor blinking. The cursor is hidden.

/** Set address pointer in CGRAM. An offset can be added:
<code>LCDCGADRSET + offset</code>. As the CGRAM is limited to 64 bytes, the
offset must not exeed 64.
*/
#define LCDCGADRSET   0x40 //0b01000000

/** Set address pointer in DDRAM. An offset can be added:
<code>LCDDDADRSET + offset</code>. As the CGRAM is limited to 128 bytes, the
offset must not exeed 128.
For code redability and ease of use the <code>LCDLINEx</code> command should be
sed!
*/
#define LCDDDADRSET   0x80 //0b10000000

/** these are used for the LCD init and module internally
*/
#define LCD2LINES     0x28 //0b00101000 //Set display mode to two lines.
//Bitpatters for LCDOUT
#define LCD8BITS      0x30 //0b00110000 //select 8 Bit interface
#define LCD4BITS      0x20 //0b00100000 //select 4 Bit interface


//forward declarations
void PutsLcd(char *nap);
void InitLcd(void);
void WriteLcd(unsigned char dana,unsigned char id);
void PutsLcd_P(PGM_P nap);

#define PutcLcd(c) WriteLcd(c,1)
#define InstLcd(i) WriteLcd(i,0)

#endif /*LCD_H*/

