#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "hw.h"


/***********************ZMIENNE EEPROM***********************/
u08 EEPROM eId;
u16 EEPROM eTZad;
u16 EEPROM eOpAmpKu;
float EEPROM epid[4];
/***********************ZMIENNE GLOBALNE***********************/
//Temperatura z przetwornika A/C
float ActTemp=0;

//Wzmocnienie wzmacniacza
u16 OpAmpKu;

//Wyjscie sterowania
float YOut;

//Temperatura zadana 
u16 TZad;

//Stan zalaczenia
enum {ST_ZAL,ST_STB,ST_WYL,ST_TUN};
u08 StanZal = ST_ZAL;

//Parametry regulatora PID
float pid[4];
#define _KP 0
#define _TI 1
#define _TD 2
#define _TP 3


//Typ enumeracji dla menu ustawien parametrow
enum {US_KU,US_KP,US_TI,US_TD,US_TP};


//Stan samonastrajania
u08 Err = 0;					
u08 StanTunn=0;


/***********************FUNKCJE***********************/

//Odtwarza parametry domyslne z eepromu
u08 RestoreConfigVals(void)
{
	if(eeprom_read_byte(&eId) != 'L') return -1;
	//Przywrocenie temperatury zadanej
	eeprom_read_block(&TZad,&eTZad,sizeof(TZad));
	//Przywrocenie wzmocnienia wzmacniacza	
	eeprom_read_block(&OpAmpKu,&eOpAmpKu,sizeof(OpAmpKu));
	//Przywrocenie parametrow pid
	eeprom_read_block(pid,epid,sizeof(pid));
	return 0;
}

//Przywraca parametry domyslne 
void DefaultConfigVals(void)
{
	//Parametry domyslne dla stacji SL1
	OpAmpKu = 1000;
	TZad = 200;
	pid[_KP] = 13.7671;
	pid[_TI] = 24.9;
	pid[_TD] = 3.984;
	pid[_TP] = 1.7434;
	//Przywracanie w eeprom
	eeprom_write_byte(&eId,'L');
	//Przywrocenie temperatury zadanej
	eeprom_write_block(&TZad,&eTZad,sizeof(TZad));
	//Przywrocenie wzmocnienia wzmacniacza	
	eeprom_write_block(&OpAmpKu,&eOpAmpKu,sizeof(OpAmpKu));
	//Przywrocenie parametrow pid
	eeprom_write_block(pid,epid,sizeof(pid));
}

//Pobiera wartosc z AC i przeksztalca ja na liczbe
float GetTemp(u16 Adc,float ku)
{
	//Tablica wielomianu dla termopary K
	const static float dw[] PROGMEM =
	{
		 2.508355E+01,
		 7.860106E-02,
		-2.503131E-01,
		 8.315270E-02,
		-1.228034E-02,
		 9.804036E-04,
		-4.413030E-05,
		 1.057734E-06, 
		-1.052755E-08
	};
	//Napiecie z przetwornika w mV
	float Vi = (Adc*10.0)/(4.0*ku);
	float di;
	float t = 0.0;
	for(int i=8;i>=0;i--)
	{
		memcpy_P(&di,&dw[i],sizeof(float));
		t += di;
		t *= Vi;
	}
	t+= TPOCZ0;
	return t;
}


//Funkcja glowna przedstawiajaca
void Intro(void)
{
	InstLcd(LCDLINE1);
	PutsLcd_P(PSTR("Regulator RL1"));
	InstLcd(LCDLINE2);
	PutsLcd_P(PSTR(__DATE__));
	DispTim = 255;
	while(DispTim) wdt_reset();
	InstLcd(LCDLINE1);
	PutsLcd_P(PSTR("L. Bryndza MKEiA"));
	InstLcd(LCDLINE2);
	PutsLcd_P(PSTR("lucck@loken.pl  "));
	DispTim = 255;
	while(DispTim) wdt_reset();
}


//Watek mierzacy temperatute otoczenia
void MeasureTempThread(void)
{
	u16 adc = AdcGetValue();
	if(adc!=-1) 
	{
		ActTemp = GetTemp(adc,OpAmpKu/10.0);
	}	
}
	

//Watek wyswietlajacy glowny screen
void DispMainScreen(void)
{
	//Jezeli nie minal czas wyswietlania to wyjdz
	if(DispTim) return;
	char buf[20];
	InstLcd(LCDLINE1);
	//Pisz temperature
	PutsLcd_P(PSTR("Tw="));
	dtostrf(ActTemp,3,0,buf);
	PutsLcd(buf);
	PutsLcd_P(PSTR("\337C"));	
	//Pisanie Aktualnego sterowania
	PutsLcd_P(PSTR(" P="));
	dtostrf(YOut*100.0,5,1,buf);
	PutsLcd(buf);
	//Druga linia
	InstLcd(LCDLINE2);
	//Temperatura zadana
	PutsLcd_P(PSTR("Tz="));
	dtostrf(TZad,3,0,buf);
	PutsLcd(buf);
	PutsLcd_P(PSTR("\337C "));	
	//Stan lutownicy
	switch(StanZal)
	{
	case ST_ZAL:
		PutsLcd_P(PSTR("  On   "));
		break;
	case ST_STB:
		PutsLcd_P(PSTR("  Sleep"));
		break;
	case ST_WYL:
		PutsLcd_P(PSTR("  Off  "));
		break;
	}
	//Nastepne wyswietlanie za 0,2s
	DispTim = TICKS_PER_SEC/10;
}




/*Wyswietla screen ustawien
 * ustan - ktory parametr ustawiamy
*/

void DispSetScreen(u08 ustan)
{
	static const char n1[] PROGMEM = "Setting OpAmpKu:";
	static const char n2[] PROGMEM = "Display Kp:";
	static const char n3[] PROGMEM = "Display Ti:";
	static const char n4[] PROGMEM = "Display Td:";
	static const char n5[] PROGMEM = "Display Tp:";
	static PGM_P opisy[] PROGMEM = {n1,n2,n3,n4,n5};
	PGM_P p;
	memcpy_P(&p,&opisy[ustan],sizeof(PGM_P));
	InstLcd(LCDCLEAR);
	InstLcd(LCDLINE1);
	PutsLcd_P(p);
	//Teraz wyswietlenie odpowiednej opcji
	char lcdbuf[40];
	switch(ustan)
	{
	case US_KU:
		dtostrf(OpAmpKu/10.0,5,1,lcdbuf);
		break;
	default:
		dtostrf(pid[ustan-US_KP],0,7,lcdbuf);
		break;
	}
	InstLcd(LCDLINE2);
	PutsLcd(lcdbuf);
}


/* Samonastrajanie regulatora */
void DisplaySelfTunning(void)
{
	if(DispTim) return;
	char buf[20];
	if(Err==2)
	{
		if(StanTunn == 1)
		{
			InstLcd(LCDLINE1+9);
			PutcLcd('1');
		}
		else if(StanTunn == 4)
		{
			InstLcd(LCDLINE1+9);
			PutcLcd('2');
		}
		InstLcd(LCDLINE2);
		PutsLcd_P(PSTR("Tw="));
		dtostrf(ActTemp,3,0,buf);
		PutsLcd(buf);
		PutsLcd_P(PSTR("\337C           "));
	}
	else if(Err==0)
	{
		InstLcd(LCDLINE1);
		PutsLcd_P(PSTR("Autoset Succes  "));	
		InstLcd(LCDLINE2);
		PutsLcd_P(PSTR("Any key return  "));
	}
	else if(Err==1)
	{
		InstLcd(LCDLINE1);
		PutsLcd_P(PSTR("Autoset Failed  "));
		InstLcd(LCDLINE2);
		PutsLcd_P(PSTR("Any key return  "));	
	}
	//Na drugiej linii temperatura
	
	DispTim = TICKS_PER_SEC/10;	
}


/* Watek wyswietlajacy */
void DisplayThread(void)
{
	//Stan menu glownego
	static u08 dstan = 0;
	//Stam ustawien
	static u08 ustan = 0;
	//Flaga zezwolen zapisu do eeprom
	static u08 savee = 0;
	//Zezwolenie na odczut klawisza
	if(KeyZTim) key_p = 0;
	//Czesc sterujaca
	switch(dstan)
	{
	//Watek wyswietlania klawiatury
	case 0:
		//Wyswietl temperature
		DispMainScreen();
		if(key_p==KEYPL)
		{
			//Klawisz +
			if(key_r) TZad += 5;
			else TZad += 1;
			if(TZad>450) TZad=450;
			Tim = TICKS_PER_SEC*2;
			savee = 1;
		}
		if(key_p==KEYMI)
		{
			//Klawisz -
			if(key_r) TZad -= 5;
			else TZad -= 1;
			if(TZad<80) TZad=80;
			Tim = TICKS_PER_SEC*2;
			savee = 1;
		}
		if(key_p==KEYMN)
		{
			//Klawisz M (zmiana trybu)
			if((StanZal+=1)>ST_WYL) StanZal=ST_ZAL;
		}
		if(key_p==(KEYMN|KEYPL))
		{
		 	//Stan ustawien
		 	dstan=1;
		 	ustan = 0;
		 	ExitTim = EXITMENUTIM * TICKS_PER_SEC;
		 	KeyZTim = TICKS_PER_SEC;
		 	DispSetScreen(ustan);
		}
		if(key_p==(KEYMN|KEYMI))
		{
			//Stan samonastrajania
			dstan = 2;
			StanZal = ST_TUN;
			StanTunn = 0;
			KeyZTim = TICKS_PER_SEC;
			InstLcd(LCDCLEAR);
			InstLcd(LCDLINE1);
			PutsLcd_P(PSTR("Autoset F"));
		}
		key_p = 0;
	break;
	//Stan menu ustawien 
	case 1:
		if(key_p==KEYMN)
		{
			if(++ustan>US_TP) ustan = US_KU;			
			DispSetScreen(ustan);
		}
		if(key_p==KEYPL && ustan==US_KU)
		{
			
			//Klawisz +
			if(key_r) OpAmpKu += 10;
			else OpAmpKu += 1;
			if(OpAmpKu>5000) OpAmpKu=5000;
			Tim = TICKS_PER_SEC*2;
			savee = 2;
			DispSetScreen(ustan);
		}
		if(key_p==KEYMI && ustan==US_KU)
		{
			//Klawisz -
			if(key_r) OpAmpKu -= 10;
			else OpAmpKu -= 1;
			if(OpAmpKu<200) OpAmpKu=200;
			Tim = TICKS_PER_SEC*2;
			savee = 2;
			DispSetScreen(ustan);
		}
		if(key_p==(KEYMN|KEYPL) || !ExitTim)
		{
			dstan = 0;
		}
		if(key_p==(KEYMN|KEYPL))
		{
			KeyZTim = TICKS_PER_SEC;
		}
		if(key_p)
		{
			//Jezeli dowolny klawisz to ustaw timer
		 	ExitTim = EXITMENUTIM * TICKS_PER_SEC;
		}
		key_p = 0;
	break;
	//Stan samostrojenia
	case 2:
		DisplaySelfTunning();
		if(key_p!=0)
		{
			dstan = 0;
			StanZal = ST_WYL;
			StanTunn = 0;
			KeyZTim = TICKS_PER_SEC;
		}
		key_p = 0;	
	break;
	}
	//Zapis do EEPROM
	if((savee==1)&&!Tim)
	{
		//Zapis do pamieci EEPROM po 2s
		eeprom_write_word(&eTZad,TZad);
		savee = 0;
	}
	else if((savee==2)&&!Tim)
	{
		//Zapis do pamieci EEPROM po 2s
		eeprom_write_block(&OpAmpKu,&eOpAmpKu,sizeof(OpAmpKu));
		savee = 0;
	}
}

//Przywrocenie wart poczatkowych
void Restore(void)
{
	if(RestoreConfigVals() || key_p==(KEYPL|KEYMN|KEYMI))
	{
		wdt_reset();
		//Tutaj blad przywrocenia z eeprom
		DefaultConfigVals();
		wdt_reset();
		//Teraz napis
		InstLcd(LCDLINE1);
		PutsLcd_P(PSTR("EEPROM error ..."));
		InstLcd(LCDLINE2);
		PutsLcd_P(PSTR("restore def. val"));
		DispTim = 255;
		while(DispTim) wdt_reset();
	}
}



//Funkcja Regulatora PID
float RegPID(float e)
{
	static float sum=0,ep=0;
	float u;
	static u08 ov = 0;
	u = pid[_KP] * e;
	if(!ov) sum += e;
	u += pid[_KP] * (pid[_TP]/pid[_TI]) * sum;
	u += pid[_KP] * (pid[_TD]/pid[_TP]) * (e - ep);
	ep = e;
	ov = 0;
	if(u > 1.0) {u = 1.0; ov = 1;}
	else if(u < 0.0) {u = 0.0; ov = 1;}
	return u;
}


#define MAX_TUNE_TIME (120*TICKS_PER_SEC)

//Samonastrajanie...
//0 - Samonastrajanie zakonczone powodzeniem
//1 - Wystapil Blad Przy samonastrajaniu
//2 - Samonastrajanie w trakcie
unsigned char SelfTunning(unsigned char *stan)
{
	static float BegTemp;					//Komorka Tymczasowa Temperatury
	static float TPocz,TDelay,T,K;
	u32 timt;
	switch(*stan)
	{
	//Stan Poczatkowy.. Wartosci poczatkowe Temperatury i czasu...
	case 0:
		cli();
		TimPid = MAX_TUNE_TIME;
		sei();
		pwm_p = 64;
		BegTemp = ActTemp;
		TPocz = ActTemp;
		TDelay = T = K = 0;
		*stan = 1; 
		break;
	//Wyznaczanie czasu Opoznienia
	case 1:
		if(!TimPid) {*stan = 2;}	//Jezeli czas sie skonczyl
		else if(ActTemp + 4 < BegTemp) 
		{
			 *stan = 2; 
		}
		else if(ActTemp > BegTemp + 8)
		{
			cli();
			timt = TimPid;
			sei();
			TDelay = ((float)(MAX_TUNE_TIME - timt))/TICKS_PER_SEC;
			*stan = 4;
			timt = TDelay * TICKS_PER_SEC * 2;
			cli();
			TimPid = timt;
			sei();
			T = 0;
			BegTemp = ActTemp;
		}
		break;
	//Wyjscie z bledem
	case 2:
		pwm_p = 0;					//Wylacz PWM
		TimPid = 0;					//Skasuj Timer
		return 1;
	//Wyjscie bez bledu
	case 3:
		pwm_p = 0;
		T = T - 3 * TDelay;				//Wyznaczenie Czasu T obiektu
		K = (K/450.0 - TPocz/450.0)/(64.0/255.0);	//Wzm Ob
		pid[_TP] = T * 0.005 + 0.1;		//Czas Probkowania Regulatora
		pid[_TI] = 2.5 * TDelay;		//Czas Calkowania Regulatora
		pid[_TD] = 0.4 * TDelay;		//Czas rozniczkowania
		pid[_KP] = T/(K * TDelay);		//Wzmocnienie Regulatora
		eeprom_write_block(pid,epid,sizeof(epid));
		TimPid = 0;						//Wyzeruj Timer
		return 0;
	//Znalezienie czasu T i wzmocnienia	
	case 4:
		if(!TimPid)
		{
			T += TDelay * 2;
			if(ActTemp + 5 < BegTemp) *stan = 2;
			else if(ActTemp <= BegTemp+4)
			{
				K = ActTemp;
				*stan = 3;
			}
			else
			{
				timt = TDelay * TICKS_PER_SEC * 2;
				cli();
				TimPid = timt;
				sei();
				BegTemp = ActTemp;
			}
		}
		break;
 	}
	
	return 2;
}

//Watek Regulacji Temperatury..
void RegThread(void)
{
		float e;				//Uchyb
		float TStab;			//Temperatura do stablizacji
		u32 timt;
		/* W zaleznosci od tego czy lutownica dziala
		 * normalnie czy zostala przelaczona w stan 
		 * standby to wybieramy albo temp cala albo polowe temp
		 */
		TStab=(StanZal==ST_STB)?(TZad/2.0):(TZad);
		
		switch(StanZal)
		{
		//Samonastrajanie
		case ST_TUN:
			Err = SelfTunning(&StanTunn);			
			if(Err != 2) StanZal= ST_WYL;
			break;
		//Wylaczona Regulacja
		case ST_WYL:
			pwm_p = 0;
			YOut = 0;
			break;
		//Zalaczony Regulator
		case ST_ZAL:
		case ST_STB:
			if(!TimPid)
			{
				timt = pid[_TP] * TICKS_PER_SEC; 
				cli();
				TimPid = timt;
				sei();
				e = (TStab/450.0) - (ActTemp/450.0);
				YOut = RegPID(e);
				pwm_p = YOut * 255;	
			}
			break;

		}
}


//Funkcja glowna main
int main(void)
{
	//Inicjalizacja watchdoga 
	wdt_enable(WDTO_1S);
	//Ustawienie trybu uspienia na idle
	set_sleep_mode(SLEEP_MODE_IDLE);
	//Inicjalizacja sprzetu
	InitHw();
	//Reset watchdoga
	wdt_reset();
	//Inicjalizacja wyswietlacza LCD
	InitLcd();
	//Reset watchdoga
	wdt_reset();
	//Glowne Intro przedstawiajace
	Intro();
	//Przywrocenie wartosci pocz z eeprom
	Restore();
	//Reset watchdoga
	wdt_reset();
	//Zeruj wcisniety klawisz
	key_p = 0;
	//Glowna petla
	while(1)
	{
		//Reset watchdoga
		wdt_reset();
		//Odczyt temperatury
		MeasureTempThread();
		//Reset watchdoga
		wdt_reset();
		//Watek Modbus
		RegThread();
		//Reset watchdoga
		wdt_reset();
		//Watek wyswietlania i klawiatury
		DisplayThread();
		//Reset watchdoga
		wdt_reset();
		//Uspienie procka
		sleep_mode();
	}
}	
